//  These are all low-level routines used by all animations.

// note that main loop doesn't make any direct referral to FastLED instructons
// they have all been encapsulated in my own functions and color definitions

// sets max brightness of all LEDs.
void brightness(int bright){  // nominal is 25
  FastLED.setBrightness(bright);
}

// copies the content of each ring the ring below it
// top ring is cleared.
void copyDown() {
  for (int i = 0; i < 24; i++) {
    for (int j = 0; j < 11; j++) {
      if (j > 5) {
        ledup[(j - 6) * 24 + i] = ledup[(j - 6) * 24 + i + 24];
      }
      else if (j < 5) {
        leddown[j * 24 + i] = leddown[j * 24 + i + 24];
      }
      else {  // j=5
        leddown[j * 24 + i] = ledup[(j - 6) * 24 + i + 24];
      }
    }
  }
}

// sets a whole ring to a hue
void setRing(int ring, byte hue) {
  for (int k = 0; k < 24; k++) {
    setHue(ring, k, hue);
  }
}

// sets a whold vertical column to a hue
void setCol(int col, byte hue) {
  for (int k = 0; k < 12; k++) {
    setHue(k, col, hue);
  }
}
// generates a wide variety of rotating spirals
void spiral(int degree, CRGB mycolor, int rotations, int mydelay, byte rotdir, byte spirdir){
  for (int ring=0; ring<12; ring++){
    if (spirdir) setStd(11-ring, ring*degree/2, mycolor);
    else setStd(ring, ring*degree/2, mycolor);
  }
   for (int k = 0; k < rotations*24; k++) {
    for (int j = 0; j < 12; j++) {
      rotate(j, rotdir); 
    }
    showKeep(mydelay);
  }
  clearAll();
}

// fades everything at a rate determined by x, the closer it is to 255, the slower the fade
void fadeAll(byte x) {
  for (int i = 0; i < NUM_LEDS; i++) {
    leddown[i].nscale8(x);
    ledup[i].nscale8(x);
  }
}

// sets an LED to a color of the rainbow from 0 - 255
// specifies an LED by its row (ring) and column (position in that circle)
void setHue(byte row, byte column, byte hue) {
  if (row < 6) leddown[row * 24 + column] = CHSV(hue, 255, 255);
  else ledup[(row - 6) * 24 + column] = CHSV(hue, 255, 255);
}

// sets an LED to one of the standard colors defined by my palette
// specifies an LED by its row (ring) and column (position in that circle
void setStd(byte row, byte column, CRGB mycolor) {
  if (row < 6) leddown[row * 24 + column] = mycolor;
  else ledup[(row - 6) * 24 + column] = mycolor;
}

// rotates the content of one ring 1 step forward (1) or backward (0)
void rotate(int row, bool Forward) {
  CRGB temp;
  if (Forward) {
    if (row < 6) {
      temp = leddown[row * 24 + 0];
      for (int j = 1; j < 24; j++) {
        leddown[row * 24 + j - 1] = leddown[row * 24 + j];
      }
      leddown[row * 24 + 23] = temp;
    }
    else {
      temp = ledup[(row - 6) * 24 + 0];
      for (int j = 1; j < 24; j++) {
        ledup[(row - 6) * 24 + j - 1] = ledup[(row - 6) * 24 + j];
      }
      ledup[(row - 6) * 24 + 23] = temp;
    }
  }
  else {
    if (row < 6) {
      temp = leddown[row * 24 + 23];
      for (int j = 22; j > -1; j--) {
        leddown[row * 24 + j + 1] = leddown[row * 24 + j];
      }
      leddown[row * 24 + 0] = temp;
    }
    else {
      temp = ledup[(row - 6) * 24 + 23];
      for (int j = 22; j > -1; j--) {
        ledup[(row - 6) * 24 + j + 1] = ledup[(row - 6) * 24 + j];
      }
      ledup[(row - 6) * 24 + 0] = temp;
    }
  }
}


// shows for x ms and doesn't clear array
void showKeep(int mydelay) {
  FastLED.show();
  delay(mydelay);
}

// shows for x ms, then clear array
void showClear(int mydelay) {
  FastLED.show();
  delay(mydelay);
  clearAll();
}

// sets all LEDs to one hue for x ms without effecting the array at all
void showAll(int mydelay, byte hue) {
  FastLED.showColor(CHSV(hue, 255, 255));
  delay(mydelay);
}

 // similar to showAll, except it does set the array to one color, and no time delay
void fillTower(byte hue) {
  for ( int i = 0; i < 144; i++) {
    leddown[i] = CHSV(hue, 255, 255);
    ledup[i] = CHSV(hue, 255, 255);
  }
}

// turn all LEDs off
void clearAll() {
  FastLED.clear();  // clears leds array
  FastLED.show();
}

// turn everything off and wait 1 sec.
void pause() {
  clearAll();
  delay(500);
}

  // Reset has been added to clean up memory allocation.  Somewhere in my code, the stack 
  // is not emptying, so repeating without reset eventually causes RAM to max out!
  // Not as good as finding the problem, but it works!
void softwareReset() {
  wdt_enable(WDTO_60MS);
  while(1) {}
}
